"""

Created on Various Days 

@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
this is atest massafe

"""

from topspy.modeling.GmTVector import *
 
    
    
def eleAxialForce(eleTag):
    import openseespy.opensees as ops
    
    '''
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
    
    This Function Return the Axial force of an element
    Tere is some Possibility that Element has some external
    Loads among its length so the axial force in the start 
    and in the end of the element can be different
    so this code returns two values, One for start and One for End
    
    Example:
            [F1, F2]=eleAxialForce(eleTag)
    
    So the F1 and F2 are the axial load of the element with the tag equal to eleTag
    '''
    Nodes=ops.eleNodes(eleTag)
    Nodei=ops.nodeCoord(Nodes[0])
    Nodej=ops.nodeCoord(Nodes[1])

    #print(f'node 1 {Nodei} and type is {type(Nodei)}')

    #Initial Data
    x1, y1, z1=Nodei[0],Nodei[1],Nodei[2]
    x2, y2, z2=Nodej[0],Nodej[1],Nodej[2]
    
    R1=[x2-x1,y2-y1,z2-z1]
    R1=Nrmlz(R1,1)

    elfor=ops.eleForce(eleTag)
    elefocesi=elfor[0:3]
    elefocesj=elfor[6:9]

    #print(f'node 1 {elefocesi} and type is {type(elefocesj)}')

    eleAxial1=DotPrdct(elefocesi,R1)
    eleAxial2=DotPrdct(elefocesj,R1)

    return eleAxial1,eleAxial2



"""

Created on Various Days 

@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)

"""

  
from topspy.modeling.GmTVector import *

def ElePerPend(Nodei,Nodej):
    
    '''
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
    
    This Function Provide A Perpendicular Vector on the Line connected from 
    Nodei to Node join
    
        Nodei : Opensees 3D nodeTag or a list with 3 values of coordinate of Nodei
        
        Nodej : Opensees 3D nodeTag or a list with 3 values of coordinate of Nodej
    
    The 3rd coordinate value is considered as Vertical Direction
    
    Example of usage:
         
          Vect=ElePerPend(Nodei,Nodej)
    
    So the Vect is a list with 3 elements that represent a vector that is perpendicular to the
    line that is tarted from nodei and end to nodej
    
    '''
    import openseespy.opensees as ops
    
    #Initial Data
    if type(Nodei)==int:
      x1, y1, z1=ops.nodeCoord(Nodei)
    else:
      x1, y1, z1=Nodei[0],Nodei[1],Nodei[2]
    
    if type(Nodej)==int:
      x2, y2, z2=ops.nodeCoord(Nodej)
    else:
      x2, y2, z2=Nodej[0],Nodej[1],Nodej[2]
    
    
    R1=[x2-x1,y2-y1,z2-z1]
    if R1[0]==0 and R1[1]==0:
       R2=[R1[0],R1[1]+0.1,R1[2]]
    else:
       R2=[R1[0],R1[1],R1[2]+0.1]
       
    
    R3=VecProduct(R1,R2)
    R3=Nrmlz(R3,1)
    
    return R3
    
 



from topspy.modeling.ElePerPend import *
import math

'''
DATE: 10/26/2021
@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
'''

def _getNewNodeNum(Nodei,Nodej):
    import openseespy.opensees as ops
  
    Nodelist=ops.getNodeTags()
    
    cnt=0

    Numb=int(str(Nodei) + str(Nodej) + str(cnt))

    check=None

    while check!=True:

        if Numb in Nodelist:
            cnt +=1
            Numb=int(str(Nodei) + str(Nodej) + str(cnt))
        else:
            check=True
            return Numb

def _getNewEleNum(Nodei,Nodej):
    import openseespy.opensees as ops
  
    EleList=ops.getEleTags()    
    cnt=0

    Numb=int(str(Nodei) + str(Nodej) + str(cnt))

    check=None

    while check!=True:

        if Numb in EleList:
            cnt +=1
            Numb=int(str(Nodei) + str(Nodej) + str(cnt))
        else:
            check=True
            return Numb
    

def MultiEl(Nodei,Nodej,Number_Of_Elements,EleParameters,MidCurveDisp=0,EndPinned='No',E=1e9,NewMaterialTag=1):
    

    '''
    
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
    
        
    This Function generate muli elements between two nodes automatically and finally after
    generating the fils it returns the middle node tag/s and it's/their coordinate. 
    
    
    Parameters
    ----------
    Acceleration : should be a List.
    Nodei              : The first node TAG.
    Nodej              : The last node TAG.
    MidCurveDisp       : By entering any value, Element will have a displacement in its center point with a sinosure shape 
    EndPinned          : If the User wants end pinned connection it Should be enter 'Yes'.
    E=1e9              : For end Pinned condition The Rigid zerolink element stiffness should be defined by E (Default=1e9)
    NewMaterialTag=1   : For End Pinned Condition a New Tag for an Elastic Material Should be assigned by user THAT Has Not Been Assigned PREVIOUSly.
    Number_Of_Elements : Number of Elements that want to be generated along node Nodei to Nodej.
    Attention:    
    *EleParameters     : is a list that all the parameters that you have to define for any kind of elements that you need
    Attention that the function automatically assign value to EleParameters[1] and EleParameters[2] and any 
    value that you enter for them is not important. for example:
    
    EleP=['elasticBeamColumn', eleTag, *eleNodes, Area, E_mod, G_mod, Jxx, Iy, Iz, transfTag]-->
    MultiEl(Nodei=1,Nodej=2,Number_Of_Elements=5,*EleParameters=EleP)
    A complete usage sample:
        
    import topspy.modeling as bjm
    transfTag +=1
    Node1=ops.nodeCoord(1)
    Node2=ops.nodeCoord(3)
    vecxz=GmT.GmTVector(Node1,Node2,Theta)
    ops.geomTransf('Linear', transfTag, *vecxz)
    elep=['elasticBeamColumn', eleTag, *eleNodes, Area, E_mod, G_mod, Jxx, Iy, Iz, transfTag] #Opensees desire Elements Parameters
    bjm.MultiEl(1,3,4,elep)
    
    Or you can write
    
    bjm.MultiEl(1,3,4,['elasticBeamColumn', eleTag, *eleNodes, Area, E_mod, G_mod, Jxx, Iy, Iz, transfTag])
    
    Also Above Codes can be written in this way: bjm.MultiEl(1,3,4,elep,MidCurveDisp=0,EndPinned='Yes',E=1e9,NewMaterialTag=10) That the last 'Yes' cause generating element with 2 end pinned head with zero length element with New matrial Tag equal to 10.
    
    Function returns:
    ----------
    midtag=list of middle point/s Tag
    midcoord=list of nodecoordinate of middle point/s
    
    '''

    
    import openseespy.opensees as ops
    
    n=int(Number_Of_Elements)
    
    #Parameters For end Pinned Material--------
    if EndPinned.upper()=='YES':
         matTag=NewMaterialTag
         try:
            ops.uniaxialMaterial('Elastic', matTag, E)   #if code encounter with error means this material has been defined previously
         except:
            pass
        

    
    #Check We will have One middle point or two and Curve Factor-----------------------------------------------
    midtag=[]
    midcoord=[]
    if n%2==0:
        elemtag=[n/2-1] #tag of element that it's 2ndNode is the middle
        
        #For Curved Case, The Perpendicular Normal vector Factors 
        fact=[math.sin(i/(n/2)*math.pi/2)*MidCurveDisp for i in range(1,int(n/2)+1)]
        fact=fact+fact[-2::-1]+[0]
        fact=[fact[0]]+[fact[i+1]-fact[i] for i in range(0,len(fact)-1)] #Increase Factore Respect to Previouse Factor
        
    else:
        elemtag=[(n-1)/2-1, (n-1)/2] #tag of element2 that it's 2ndNodes are the middle
        
        #For Curved Case, The Perpendicular Normal vector Factors
        fact=[math.sin(i/((n-1)/2)*math.pi/2)*MidCurveDisp for i in range(1,int((n-1)/2)+1)]
        fact=fact+fact[-1::-1]+[0]
        fact=[fact[0]]+[fact[i+1]-fact[i] for i in range(0,len(fact)-1)] #Increase Factore Respect to Previouse Factor
        
    
    
    
    NPV=ElePerPend(Nodei,Nodej) #Perpendicular direction Normal Vector
        
    
    
    NodeiC=ops.nodeCoord(Nodei)
    NodejC=ops.nodeCoord(Nodej)

    x1,y1,z1=NodeiC[0],NodeiC[1],NodeiC[2]
    x2,y2,z2=NodejC[0],NodejC[1],NodejC[2]
    Lxi=(x2-x1)/n
    Lyi=(y2-y1)/n
    Lzi=(z2-z1)/n


    
    FstNode=Nodei #First Node Tag
    
    
    
    #First zerolength Element
    if EndPinned.upper()=='YES':    #If user Decide to have an element with end pinned connection-----------------
        
        Fstcoord=ops.nodeCoord(Nodei)
        lastcoord=ops.nodeCoord(Nodej)
        vecx=[lastcoord[0]-Fstcoord[0],lastcoord[1]-Fstcoord[1],lastcoord[2]-Fstcoord[2]] #Longitudinal Vector of the Element
        if vecx[0]==0 and vecx[2]==0:
            vecyp=[vecx[0]+0.1,vecx[1],vecx[2]]
        else:
            vecyp=[vecx[0],vecx[1]+0.1,vecx[2]]
        
        FstNode=_getNewNodeNum(Nodei,Nodej)
        ops.node(FstNode,*Fstcoord)
        
        Newele=_getNewEleNum(Nodei,Nodej) # New element Tag
        
        ops.element('zeroLength', Newele, *[Nodei,FstNode], 
                    '-mat', *[matTag,matTag,matTag,matTag],
                    '-dir', *[1,2,3,4],
                    '-orient', *vecx, *vecyp)
     #-------------------------------------------------------------------------------------------------------------   
        
    # --- Generating Nodes and Elements----------------------------------------------------------------------------
    for i in range(n):
        
        Fact=fact[i]   #Factor of curved element
        
        Fstcoord=ops.nodeCoord(FstNode) #Get First Node Coordinate
        Sndcoord=[Fstcoord[0]+Lxi,Fstcoord[1]+Lyi,Fstcoord[2]+Lzi] #Second Node Coordinate
        
        Sndcoord=[i+Fact*j for i,j in zip(Sndcoord,NPV)] #Add the Curved Value to the node coordinate Using Fact and NPV
        
        if i==n-1 and EndPinned.upper()!='YES': #If second node is comatible on the last node
            SndNode=Nodej                       #Second node is the last node and no need to produce new node
            Sndcoord=ops.nodeCoord(SndNode)
        
        else:

            SndNode=_getNewNodeNum(Nodei,Nodej) 
            ops.node(SndNode,*Sndcoord)
            
            if i in elemtag:
                midtag.append(SndNode)
                midcoord.append(Sndcoord)
        
        Newele=_getNewEleNum(Nodei,Nodej) # New element Tag

        eleTag=Newele
        eleNodes=[FstNode,SndNode]+ EleParameters[4:]
        ops.element(EleParameters[0], eleTag, *eleNodes)


        FstNode=SndNode
        
        #Last Zero Length Element---------------------------------------------------------------
        if i==n-1 and EndPinned.upper()=='YES': #If second node is comatible on the last node

            Fstcoord=ops.nodeCoord(Nodei)
            lastcoord=ops.nodeCoord(Nodej)
            
            vecx=[lastcoord[0]-Fstcoord[0],lastcoord[1]-Fstcoord[1],lastcoord[2]-Fstcoord[2]] #Longitudinal Vector of the Element
            if vecx[0]==0 and vecx[2]==0:
                vecyp=[vecx[0]+0.1,vecx[1],vecx[2]]
            else:
                vecyp=[vecx[0],vecx[1]+0.1,vecx[2]]
            
            Sndcoord=ops.nodeCoord(SndNode)
            
            Newele=_getNewEleNum(Nodei,Nodej) # New element Tag
            
            ops.element('zeroLength', Newele, *[SndNode,Nodej],
                        '-mat', *[matTag,matTag,matTag],
                        '-dir', *[1,2,3],
                        '-orient', *vecx, *vecyp)
            #---------------------------------------------------------------------------------------



    return midtag, midcoord





####################################  Second Format Using EqualDOF ###############################################################

def MultiElEQDOF(Nodei,Nodej,Number_Of_Elements,EleParameters,MidCurveDisp=0,EndPinned='No'):
    

    '''
    
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
    
        
    This Function generate muli elements between two nodes automatically and finally after
    generating the fils it returns the middle node tag/s and it's/their coordinate. 
    
        **ATTENTION: The difference of this Function With Previous one is in the Type of Considering
                 End Pinned Condition. In this Function Using EQUALDOF the end Pinned Condition 
                 Has been Considered**
    
    
    Parameters
    ----------
    Acceleration : should be a List.
    Nodei              : The first node TAG.
    Nodej              : The last node TAG.
    MidCurveDisp       : By entering any value, Element will have a displacement in its center point with a sinosure shape 
    EndPinned          : If the User wants end pinned connection it Should be enter 'Yes'.
    E=1e9              : For end Pinned condition The Rigid zerolink element stiffness should be defined by E (Default=1e9)
    NewMaterialTag=1   : For End Pinned Condition a New Tag for an Elastic Material Should be assigned by user THAT Has Not Been Assigned PREVIOUSly.
    Number_Of_Elements : Number of Elements that want to be generated along node Nodei to Nodej.
    Attention:    
    *EleParameters     : is a list that all the parameters that you have to define for any kind of elements that you need
    Attention that the function automatically assign value to EleParameters[1] and EleParameters[2] and any 
    value that you enter for them is not important. for example:
    
    EleP=['elasticBeamColumn', eleTag, *eleNodes, Area, E_mod, G_mod, Jxx, Iy, Iz, transfTag]-->
    MultiEl(Nodei=1,Nodej=2,Number_Of_Elements=5,*EleParameters=EleP)
    A complete usage sample:
        
    import topspy.modeling as bjm
    transfTag +=1
    Node1=ops.nodeCoord(1)
    Node2=ops.nodeCoord(3)
    vecxz=GmT.GmTVector(Node1,Node2,Theta)
    ops.geomTransf('Linear', transfTag, *vecxz)
    elep=['elasticBeamColumn', eleTag, *eleNodes, Area, E_mod, G_mod, Jxx, Iy, Iz, transfTag] #Opensees desire Elements Parameters
    bjm.MultiEl(1,3,4,elep)
    
    Or you can write
    
    bjm.MultiEl(1,3,4,['elasticBeamColumn', eleTag, *eleNodes, Area, E_mod, G_mod, Jxx, Iy, Iz, transfTag])
    
    Also Above Codes can be written in this way: bjm.MultiEl(1,3,4,elep,MidCurveDisp=0,EndPinned='Yes',E=1e9,NewMaterialTag=10) That the last 'Yes' cause generating element with 2 end pinned head with zero length element with New matrial Tag equal to 10.
    
    Function returns:
    ----------
    midtag=list of middle point/s Tag
    midcoord=list of nodecoordinate of middle point/s
    
    '''

    
    import openseespy.opensees as ops
    
    n=int(Number_Of_Elements)
    
   
    #Check We will have One middle point or two and Curve Factor-----------------------------------------------
    midtag=[]
    midcoord=[]
    if n%2==0:
        elemtag=[n/2-1] #tag of element that it's 2ndNode is the middle
        
        #For Curved Case, The Perpendicular Normal vector Factors 
        fact=[math.sin(i/(n/2)*math.pi/2)*MidCurveDisp for i in range(1,int(n/2)+1)]
        fact=fact+fact[-2::-1]+[0]
        fact=[fact[0]]+[fact[i+1]-fact[i] for i in range(0,len(fact)-1)] #Increase Factore Respect to Previouse Factor
        
    else:
        elemtag=[(n-1)/2-1, (n-1)/2] #tag of element2 that it's 2ndNodes are the middle
        
        #For Curved Case, The Perpendicular Normal vector Factors
        fact=[math.sin(i/((n-1)/2)*math.pi/2)*MidCurveDisp for i in range(1,int((n-1)/2)+1)]
        fact=fact+fact[-1::-1]+[0]
        fact=[fact[0]]+[fact[i+1]-fact[i] for i in range(0,len(fact)-1)] #Increase Factore Respect to Previouse Factor
        
    
    
    
    NPV=ElePerPend(Nodei,Nodej) #Perpendicular direction Normal Vector
        
    
    
    NodeiC=ops.nodeCoord(Nodei)
    NodejC=ops.nodeCoord(Nodej)

    x1,y1,z1=NodeiC[0],NodeiC[1],NodeiC[2]
    x2,y2,z2=NodejC[0],NodejC[1],NodejC[2]
    Lxi=(x2-x1)/n
    Lyi=(y2-y1)/n
    Lzi=(z2-z1)/n


    
    FstNode=Nodei #First Node Tag
    
    if EndPinned.upper()=='YES':    #If user Decide to have an element with end pinned connection
        Fstcoord=ops.nodeCoord(FstNode)
        FstNode=_getNewNodeNum(Nodei,Nodej)
        ops.node(FstNode,*Fstcoord)
        ops.equalDOF(Nodei, FstNode, *[1,2,3])
    
  
    # --- Generating Nodes and Elements----------------------------------------------------------------------------
    for i in range(n):
        
        Fact=fact[i]   #Factor of curved element
        
        Fstcoord=ops.nodeCoord(FstNode) #Get First Node Coordinate
        Sndcoord=[Fstcoord[0]+Lxi,Fstcoord[1]+Lyi,Fstcoord[2]+Lzi] #Second Node Coordinate
        
        Sndcoord=[i+Fact*j for i,j in zip(Sndcoord,NPV)] #Add the Curved Value to the node coordinate Using Fact and NPV
        
        if i==n-1: #If second node is comatible on the last node
            SndNode=Nodej                       #Second node is the last node and no need to produce new node
            Sndcoord=ops.nodeCoord(SndNode)
            
            
            if EndPinned.upper()=='YES':  #If user Decide to have an element with end pinned connection
                Sndcoord=ops.nodeCoord(SndNode)
                SndNode=_getNewNodeNum(Nodei,Nodej)
                ops.node(SndNode,*Sndcoord)
                ops.equalDOF(Nodej, SndNode, *[1,2,3])
        
        else:                              #For second node we have to define new node

            SndNode=_getNewNodeNum(Nodei,Nodej) 
            ops.node(SndNode,*Sndcoord)
            
            if i in elemtag:
                midtag.append(SndNode)
                midcoord.append(Sndcoord)
        
        Newele=_getNewEleNum(Nodei,Nodej) # New element Tag

        eleTag=Newele
        eleNodes=[FstNode,SndNode]+ EleParameters[4:]
        ops.element(EleParameters[0], eleTag, *eleNodes)


        FstNode=SndNode
        




    return midtag, midcoord


#Remember that each folder contain their own records
# and the LAT2 function inside each folder is for only available records in that folder
# nad they differ from LAT2 in Dynamic subpackage that loads ant AT2 inside the current folder
# for any new folder of records only add __init__ and LAT2.py from one of the existing folders
 


"""

Created on Various Days 

@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)

"""
   
     
def ElasticSDF(wn,Damping):
    '''
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com
    
    Parameters
    ----------
    wn : Natural period of the SDF that should be a float value.
    Damping : Is a float variable.
    
    Funtion for Modelng a SDF:
    ElasticSDF(5,0.02)
    
    '''
    import openseespy.opensees as ops
   
    
    #Build the model
    ops.wipe()
    ops.model('basic', '-ndm', 2)
        
    # Define some parameters ...   
    m=1
    k=wn**2*m
    L=1
    E=float(100)
    A=k*L/E
    
    # Define an Elastic Material
    matTag=1
    ops.uniaxialMaterial("Elastic", matTag, E)
    
    #Define 2 nodes and mass and truss element
    ops.node(1, *[0,0])
    ops.fix(1,1,1,1)
    ops.node(2, *[L,0], '-mass', *[m,m,1.0e-10])
    ops.element('Truss', 1, *[1,2], A, matTag)
    
    
    # set damping based on first eigen mode
    freq = ops.eigen('-fullGenLapack', 3) # for the define model Always there is 3 Eigen Values
    freq=max(freq)
    freq=freq**0.5
    ops.rayleigh(2*Damping*freq, 0., 0., 0)
    
    
    print(f'2DModel is ready and in the domain. node 1 is fixed and node 2 in the right side of node 1 is free and their mass and stifness cause wn={wn} or Tn={2*3.1415/wn}')


"""

Created on Various Days 

@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)

"""

def VecProduct(V1,V2):
  #Calculate Vectors External Products
  x=(V1[1]*V2[2]-V1[2]*V2[1])
  y=(-V1[0]*V2[2]+V1[2]*V2[0])
  z=(V1[0]*V2[1]-V2[0]*V1[1])
  return x,y,z

def DotPrdct(V1,V2):
    #Calculate vectors internal products    
    return V1[0]*V2[0]+V1[1]*V2[1]+V1[2]*V2[2]

def Nrmlz(V,a):
    #Normilize of vector V size equal to scalar  a
    A=(V[0]**2+V[1]**2+V[2]**2)**0.5
    V=V[0]/A*a,V[1]/A*a,V[2]/A*a
    return V

def VecSize(V):
    'Return a vector size'
    return (V[0]**2+V[1]**2+V[2]**2)**0.5

def GmTVector(FirstNode,SecondNode,Theta,VerticalDOF=3):
    
    '''
    
     @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
              Silvia Mazzoni (SilviaMazzoni@yahoo.com)
     @Date: 10/14/2021
    
    -----------------------------------------------------------------------
    Parameters:
    -----------------------------------------------------------------------
    FirstNode : Opensees 3D node as a list with 3 elements
        
    SecondNode : Opensees 3D node as a list with 3 elements

    Theta (Deg) : An scalar value that shows the cross section rotation angle.
    
    VerticalDOF : Shows the Vertical Direction that is considered by user 
                  For example if I considered nodes coordinate as x,y,z and 
                  z is my vertical Dof so I have to enter 3
                  if y is my verticalDOF then I have to enter 2
                  if x is my vertical DOF then I have to enter 1
                  
    -------------------------------------------------------------------------
    Description:
    -------------------------------------------------------------------------
    Non_Columns:
        For any Elements except columns, when you look from the Nodei (First node of element)
        to the Nodej(Second node of the element) The main local axis of the element is considered 
        To THE LEFT SIDE OF THE MENTIONED DIRECTION.
                
    
    Columns:
        For columns the cross section main axis is considered in 1st horizontal coordinate direction 
        if the first node is located above the second nodeand and for vise versa it is in the negative 
        direction of the 1st coordinate. Any theta value is the rotation clockwise
        
        
                           _____________
                           |            |
                           |            |
                           |            |
                           |   <------  |  The Main local axis of cross section.
                           |            |
                           |____________|
        

    -------------------------------------------------------------------------
    Function Returns:
    -------------------------------------------------------------------------
    GeomTrans : return a list with 3 elements that are 3 scalar values 
        using this function you will have the local Geometric transforemation of the element
        
    -------------------------------------------------------------------------
    Usage Example:
    -------------------------------------------------------------------------    
    Nodei=[5, 6, 3]
    Nodej=[8, 9, 5]
    Theta=0
    GTR=GmTVector(Nodei,Nodej,Theta,VerticalDOF=3)
    #GTR is the Geometric Transformation Vector of the element with Nodei and Nodej as its end Nodes
    
    a=GmTVector([0,0,0],[10,0,0],0) ----> a=[0.0, 1.0, 0]
    a=GmTVector([10,0,0],[0,0,0],0) ----> a=[0.0, -1.0, 0]
    a=GmTVector([0,0,0],[10,0,0],90) ----> a=[0.0, 0, -1.0]
    
    a=GmTVector([0,0,0],[0,10,0],0) ----> a=[-1.0, 0.0, 0]
    a=GmTVector([0,10,0],[0,0,0],0) ----> a=[1.0, 0.0, 0]
    a=GmTVector([0,0,0],[0,10,0],90) ----> a=[0, 0.0, -1.0]
    
    a=GmTVector([0,0,0],[0,0,10],0) ----> a=[-1.0, 0, 0.0]
    a=GmTVector([0,0,10],[0,0,0],0) ----> a=[1.0, 0, 0.0]
    '''
    
    # Convert nodeTag to Node coordinate IF USER ENTER NODE TAG
    import openseespy.opensees as ops
    if type(FirstNode)==int:
      FirstNode=ops.nodeCoord(FirstNode)
    
    if type(SecondNode)==int:
      SecondNode=ops.nodeCoord(SecondNode)
    
    
    
    
    # First, if Vertical Direction is not the 3rd vector I change the input values and move the 
    # vertical dof to the end 
    
    if VerticalDOF==3:
       pass
    elif VerticalDOF==2:
       FirstNode[1], FirstNode[2]=FirstNode[2], FirstNode[1]
       SecondNode[1], SecondNode[2]=SecondNode[2], SecondNode[1]
    elif VerticalDOF==1:
       FirstNode[0], FirstNode[2]=FirstNode[2], FirstNode[0]
       SecondNode[0], SecondNode[2]=SecondNode[2], SecondNode[0]       
    else:
       return "Wrong Value for Vertical DOF"
        
       
    #First node and second node are List by 3 elements and theta is the rotation angle
    #--------------------------Finding main Vectors -------------------
    import math
    
    #Initial Data
    x1, y1, z1=FirstNode[0],FirstNode[1],FirstNode[2]
    x2, y2, z2=SecondNode[0],SecondNode[1],SecondNode[2]
    theta=Theta-180

    
    #V1 is the Main element
    V1=[(x2-x1),(y2-y1),(z2-z1)]
    
    #V2 is a vector that is located in Perpendicular plane
    z2=z2+0.1
    
    if (x1==x2 and y1==y2): #Columns Condition
       z2=z1
       x2=x1
       y2=y1-0.1
       
    V2=[(x2-x1),(y2-y1),(z2-z1)]
     #VN is the vector that is normal to the perpendicular page of V1 that is that z local axis of section
    VN=VecProduct(V1,V2)
     #VN=Nrmlz(VN,1)
     
     #print(f'V1.VN={round(DotPrdct(V1,VN),4)}')
     #print(f'V2xVN={round(DotPrdct(V2,VN),4)}')
     
     #---------------------- Rotating -----------------------------
    theta=-theta*float(math.pi)/180
    c=math.cos(theta)
    s=math.sin(theta)
     
    #VR is rotated vector of VN around V1 (Main Vector) REF:https://en.wikipedia.org/wiki/Rotation_matrix
    ux,uy,uz=V1[0]/VecSize(V1),V1[1]/VecSize(V1),V1[2]/VecSize(V1)
    VR=[
         (c+ux**2*(1-c))*VN[0]+(ux*uy*(1-c)-uz*s)*VN[1]+(ux*uz*(1-c)+uy*s)*VN[2],
         (uy*ux*(1-c)+uz*s)*VN[0]+(c+uy**2*(1-c))*VN[1]+(uy*uz*(1-c)-ux*s)*VN[2],
         (uz*ux*(1-c)-uy*s)*VN[0]+(uz*uy*(1-c)+ux*s)*VN[1]+(c+uz**2*(1-c))*VN[2]
         ]
     
    VR=Nrmlz(VR,1)
    VN=Nrmlz(VN,1)
     
     #print (f'V1={V1} and its size is equal to {round(VecSize(V1),4)}')
     #print (f'VR={VR} and its size is equal to {round(VecSize(VR),4)}')
     #print (f'VN={VN} and its size is equal to {round(VecSize(VN),1)}')
     
     #print(f'V1.VR={round(DotPrdct(V1,VR),4)}')
     #print(f'VN.VR={round(DotPrdct(VN,VR),4)}')
     
     #--------------- Calculation Of Geometric Transform ---------------------
     
    VRsize=VecSize(VR)
     #print(f'Size of VR is {VRsize}')
     
    GeomTrans=[VR[0]/VRsize,VR[1]/VRsize,VR[2]/VRsize]
    
    
    # Becaus at the first of the code I changes the vertical DOF location, Finally
    # I have to move it back to its initial location    
    if VerticalDOF==3:
       pass
    elif VerticalDOF==2:
       GeomTrans[1], GeomTrans[2]=GeomTrans[2], GeomTrans[1]
    elif VerticalDOF==1:
       GeomTrans[0], GeomTrans[2]=GeomTrans[2], GeomTrans[0]

    
    
    
    return GeomTrans
     #print(f'GeomTransfor={GeomTrans}')



"""

@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)

"""
import os as _os
def LAT2(Filename):
    
    '''
     @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)
     
     LAT2=Load *.AT2 files in the current list
     
     A function for Transfering PEER NGA files (*.AT2) to the code parameters
     
     The function return [dt,e] where:
         
         dt= time interval (Float)
         e= Accelerogram Values (List)
     
    '''
    
    st=str(Filename)+'.AT2'
    
    pathdir=_os.path.dirname(__file__) #Get the current file path
    st=pathdir+'\\'+st
        
    
    f = open(st,'r')
    
    ee=f.readlines() #All data are located in e and so e is the time history 
    f.close()

    e=[]
    dt=None
    for i in ee:
        i=i.split()
        i=[i.upper() for i in i]
        
        #-------------------------------   Find dt ---------------------------------------------------------------------
        if 'DT=' in i:  # Find dt (Time interval)
            try:  #To check If next value doesn't be a number
                dt=float(i[i.index('DT=')+1])
            except:
                dt=None
                continue 
                
        if [a for a in i if a.startswith('DT=')==True and len(a)>3]!=[]:  #If Time Value was sticked to 'dt' Expression
            try: #To check If next value doesn't be a number
                dt=[a for a in i if a.startswith('DT=')==True and len(a)>3][0]
                dt=float(dt[3:len(dt)])
            except:
                dt=None
                continue
                
                
        if dt==None and 'NPTS,' in i and 'DT' in i:
            try:  #To check If next value doesn't be a number
                dt=float(i[1])
            except:
                dt=None
                continue
        #-----------------------------------------------------------------------------------------------------------------
        
    
        try: #Convert lines that are not string
            e.extend([float(m) for m in i])
        except:
            continue
    
    return dt,e


def Recordslist():
    pathdir=_os.path.dirname(__file__)
    return [x[:-4] for x in _os.listdir(pathdir) if x.endswith(".AT2") or x.endswith(".At2") or x.endswith(".at2")] #Get list of directories in the current file



"""

Created on Various Days 

@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)

"""
   
     
def SDFTHA(Acceleration,dt,wn,Damping):
    '''
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com
    @Date: 10/14/2021
    
    
    Parameters
    ----------
    Acceleration : should be a List.
    dt : is a float variable.
    wn : Natural period of the SDF that should be a float value.
    Damping : Is a float variable.
    
    
    Funtion for calculating time history response of a SDF
    [t,ut]=SDFTH(e,dt,5,0.02)
    #Sample usage of the above function that calculate time history displacement response of a SDF with 
    0.02 damping and natural frequency equal to 2 under an acceleration that store in e (list) with dt time interval
    
    '''
    import openseespy.opensees as ops
    import numpy as np
    
    
    #Build the model
    ops.wipe()
    ops.model('basic', '-ndm', 2)
        
    # Define some parameters ...   
    m=1
    k=wn**2*m
    L=1
    E=float(100)
    A=k*L/E
    
    # Define an Elastic Material
    matTag=1
    ops.uniaxialMaterial("Elastic", matTag, E)
    
    #Define 2 nodes and mass and truss element
    ops.node(1, *[0,0])
    ops.fix(1,1,1,1)
    ops.node(2, *[L,0], '-mass', *[m,m,1.0e-10])
    ops.element('Truss', 1, *[1,2], A, matTag)
    
    
    # set damping based on first eigen mode
    freq = ops.eigen('-fullGenLapack', 3) # for the define model Always there is 3 Eigen Values
    freq=max(freq)
    freq=freq**0.5
    ops.rayleigh(2*Damping*freq, 0., 0., 0)
    
    
    # Set time series to be passed to uniform excitation
    ops.timeSeries('Path', 1, '-dt', dt, '-values', *Acceleration)

    # Create UniformExcitation load pattern
    ops.pattern('UniformExcitation',  1,   1,  '-accel', 1)

    # Delete the old analysis and all it's component objects
    ops.wipeAnalysis()
    ops.system('SparseGEN')
    ops.constraints('Transformation')
    ops.test('EnergyIncr', 1.0e-12,  10 )
    ops.algorithm('ModifiedNewton')
    ops.numberer('RCM')
    ops.integrator('Newmark',  0.5,  0.25 )
    ops.analysis('Transient')
    
    
    ut=[]
    
    for i in Acceleration: #Run Analysis for each step and get the results 
        ops.analyze(1, dt)
        u=ops.nodeResponse(2, 1, 1)
        ut.append(u)
    
    
    t=np.arange(0,dt*len(Acceleration),dt) #Time Vector
    
    
    
    
    #slv.drawModel() #To check the defined model
    return t, ut #Return the time vector and displacement history (ut)
    #--------------------------- End of the Function --------------------------
  



"""

Created on Various Days 

@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)

"""
   
     
def NSDF(Tn, DampRatio, E, Fy):
    
    '''
    This function Models a nonlinear single degree of freedom system where Fy is the yielding force
    
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com
    @Date: 10/14/2021
    
    
    Parameters
    ----------
    Tn : Natural period of the SDF that should be a float value.
    DampRatio : Is a float variable.
    E= Stiffness of the system
    Fy=Yield Force of the system
    '''
    import openseespy.opensees as ops
    #Model
    op.wipe()
    op.model('basic', '-ndm', 2, '-ndf', 3) 
    
    # Define some parameters ...   
    wn=2*3.1415/Tn
    L=1
    A=1
    k=E*A/L
    m=k/wn**2
    
    # Define an Elastic Material
    epsyP=Fy/E
    matTag=1
    ops.uniaxialMaterial('ElasticPP', MatTag, E, epsyP)
    
    #Define 2 nodes and mass and truss element
    ops.node(1, *[0,0])
    ops.fix(1,1,1,1)
    ops.node(2, *[L,0], '-mass', *[m,m,1.0e-10])
    ops.element('Truss', 1, *[1,2], A, matTag)
    
    # set damping based on first eigen mode
    freq = ops.eigen('-fullGenLapack', 3) # for the define model Always there is 3 Eigen Values
    freq=max(freq)
    freq=freq**0.5
    ops.rayleigh(2*DampRatio*freq, 0., 0., 0)    

    print(f'2DModel is ready and in the domain. node 1 is fixed and node 2 in the right side of node 1 is free and their mass and stifness cause wn={wn} or Tn={2*3.1415/wn}')


def KMCMatrixes():
    '''
    This function returns Stiffness, Mass and Damping Matrixes for the model that is loaded in the Domain
    [K,M,C]=KMCMatrixes()
    
    ## References

    - [Prof. Michael H. Scott - Mass & Stiffness Matrixes in Opensees](https://portwooddigital.com/2020/05/17/gimme-all-your-damping-all-your-mass-and-stiffness-too/)
    '''

    import openseespy.opensees as ops
    import numpy as np
    ## Mass And Stiffness Matrixes Calculation
    # create the analysis
    ops.wipeAnalysis()    # clear previously-define analysis parameters
    ops.constraints('Plain')    # how it handles boundary conditions
    ops.numberer('Plain')    # renumber dof's to minimize band-width (optimization), if you want to
    ops.system('FullGeneral') # how to store and solve the system of equations in the analysis
    ops.algorithm('Linear') # use Linear algorithm for linear analysis
    ops.integrator('GimmeMCK',1.0,0.0,0.0)
    ops.analysis('Transient')   # define type of analysis: time-dependent
    # ok=ops.analyze(1,dt)

    # Mass
    ops.integrator('GimmeMCK',1.0,0.0,0.0)
    ops.analyze(1,0.0)

    # Number of equations in the model
    N = ops.systemSize() # Has to be done after analyze

    M = ops.printA('-ret') # Or use ops.printA('-file','M.out')
    # print('M=',M)
    M = np.array(M) # Convert the list to an array
    M.shape = (N,N) # Make the array an NxN matrix


    # Stiffness
    ops.integrator('GimmeMCK',0.0,0.0,1.0)
    ops.analyze(1,0.0)
    K = ops.printA('-ret')
    # print('K=',K)
    K = np.array(K)
    K.shape = (N,N)


    # Damping (We do not need Damping matrix. I put it here just for any possible future need)
    ops.integrator('GimmeMCK',0.0,1.0,0.0)
    ops.analyze(1,0.0)
    C = ops.printA('-ret')
    # print('C=',C)
    C = np.array(C)
    C.shape = (N,N)
    
    return K,M,C


from setuptools import setup, find_packages

import subprocess
import os

TopsPy_version = (
    subprocess.run(["git", "describe", "--tags"], stdout=subprocess.PIPE)
    .stdout.decode("utf-8")
    .strip()
)

assert "." in TopsPy_version

assert os.path.isfile("topspy/version.py")
with open("topspy/VERSION", "w", encoding="utf-8") as fh:
    fh.write(f"{TopsPy_version}\n")



with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name="TOpsPy",
    version=TopsPy_version,
    author="BijanSeif (Bijan Sayyafzadeh)",
    author_email="<b.sayyaf@yahoo.com>",
    description="OpenseesPy Functions",
    long_description_content_type="text/markdown",
    long_description=long_description ,
    package_data={
        "":["*.AT2","*.At2","*.at2"],
        # If any package contains *.txt or *.rst files, include them:
        #"": ["*.txt", "*.rst"],
        # And include any *.msg files found in the "hello" package, too:
        #"hello": ["*.msg"],
    },
    packages=find_packages(),
    install_requires=['openseespy', 'eseesminipy','numpy'],
    url="https://github.com/BijanSeif",
    keywords=['python', 'opensees', 'Modeling', 'Dynamic'],
    classifiers=[
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Operating System :: Microsoft :: Windows'
    ],

)



"""


@author: Bijan SayyafZadeh (B.sayyaf@yahoo.com)

"""
from topspy.dynamic.SDFTHA import *
 
def RspSpc(Acceleration,dt,PeriodRange,Damping):
    '''
    @author: Bijan SayyafZadeh (B.sayyaf@yahoo.com
    @Date: 10/14/2021
    
    This function return the linear response spectrum of the defined acceleration in the definded period range 

    Parameters
    ----------
    Acceleration : should be a List.
    dt : is a float variable.
    PeriodRange : Should be a list.
    Damping : Is a float variable.

    The function returns 
    -------
    Tn : Period Range as list
    d : Displacement response spectrum as list.
    v : Pseudo Velocity as list.
    s : Pseudo Acceleration as list.

    '''
    
    
    import math
    
    
    #Define some parameters to store the results
    Tn=[]
    d=[]
    v=[]
    s=[]
    
    for T in PeriodRange:
        wn=2*math.pi/T
        [t,ut]=SDFTHA(Acceleration,dt,wn,Damping)
        ut=[abs(i) for i in ut]
        d.append(max(ut))
        v.append(max(ut)*wn)
        s.append(max(ut)*wn**2)
        Tn.append(T)
     
    return Tn,d,v,s




def BuildingFrame3D(numBayX, numBayY, numFloor):
    import openseespy.opensees as ops
    '''
    This model modes 3D building that beams and columns are connectied to each other continuesly
    only enter number of bays and floors
    model is ready for eigen value analysis
    Sample:
    BuildingFrame3D(numBayX=4, numBayY=4, numFloor=5)
    
    '''
    ops.wipe()
    ops.model('Basic', '-ndm', 3, '-ndf', 6)

    bayWidthX = 6.0
    bayWidthY = 5.0
    storyHeights = [3 for x in range(numFloor+1)]

    E = 2.1e10
    massX = 2000

    coordTransf = "Linear"


    # Nodes
    nodeTag = 1
    zLoc = 0.
    for k in range(0, numFloor + 1):
        xLoc = 0. 
        for i in range(0, numBayX + 1):
            yLoc = 0.
            for j in range(0, numBayY + 1):
                ops.node(nodeTag, xLoc, yLoc, zLoc)
                ops.mass(nodeTag, massX, massX, 0.01, 1e-5,1e-5, 1e-5)
                if k == 0:
                    ops.fix(nodeTag, 1, 1, 1, 1, 1, 1)

                yLoc += bayWidthY
                nodeTag += 1
                NodetagL=nodeTag-1

            xLoc += bayWidthX

        if k < numFloor:
            storyHeight = storyHeights[k]

        zLoc += storyHeight


    # Define Elements
    ops.geomTransf("Linear", 1, 1, 0, 0)
    ops.geomTransf("Linear", 2, 0, 0, 1)

    eleTag = 1
    nodeTag1 = 1

    #Add Columns
    for k in range(0, numFloor):
        for i in range(0, numBayX+1):
            for j in range(0, numBayY+1):
                nodeTag2 = nodeTag1 + (numBayX+1)*(numBayY+1)
                iNode = ops.nodeCoord(nodeTag1)
                jNode = ops.nodeCoord(nodeTag2)
                ops.element('elasticBeamColumn', eleTag, nodeTag1, nodeTag2, 0.25, E, E/2/1.3, 0.1**4/12, 0.05**4/12, 0.05**4/12, 1)
                eleTag += 1
                nodeTag1 += 1

    # add beam elements
    nodeTag1 = 1+ (numBayX+1)*(numBayY+1)

    for j in range(1, numFloor + 1):
        for i in range(0, numBayX):
            for k in range(0, numBayY+1):
                nodeTag2 = nodeTag1 + (numBayY+1)
                iNode = ops.nodeCoord(nodeTag1)
                jNode = ops.nodeCoord(nodeTag2)
                ops.element('elasticBeamColumn', eleTag, nodeTag1, nodeTag2, 0.25, E, E/2/1.3, 0.1**4/12, 0.05**4/12, 0.05**4/12, 2)
                eleTag += 1
                nodeTag1 += 1

        nodeTag1 += (numBayY+1)

    # add beam elements
    nodeTag1 = 1+ (numBayX+1)*(numBayY+1)

    for j in range(1, numFloor + 1):
        for i in range(0, numBayY+1):
            for k in range(0, numBayX):
                nodeTag2 = nodeTag1 + 1
                iNode = ops.nodeCoord(nodeTag1)
                jNode = ops.nodeCoord(nodeTag2)
                ops.element('elasticBeamColumn', eleTag, nodeTag1, nodeTag2, 0.25, E, E/2/1.3, 0.1**4/12, 0.05**4/12, 0.05**4/12, 2)
                eleTag += 1
                nodeTag1 += 1
            nodeTag1 += 1

    # set damping based on first eigen mode
    # freq = ops.eigen('-genBandArpack', 1)[0]**0.5
    # dampRatio = 0.05
    # ops.rayleigh(0., 0., 0., 2*dampRatio/freq)